import java.io.IOException;
import java.io.PrintWriter;
import java.io.OutputStream;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Random;
import java.util.TimeZone;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.aspose.cells.*;

public class ChartDemoServlet extends HttpServlet
{
   /**
    *
    */
   private static final long serialVersionUID = 3257562889065082933L;
   private static final String DB_URL_P1 = "jdbc:odbc:driver={Microsoft Access Driver (*.mdb)};DBQ=";
   private static final String DB_URL_P2 = "/WEB-INF/Database/Northwind.mdb";

   private static final String TEMPLATE_FILE_PATH_PART = "/WEB-INF/Designer/Northwind.xls";

   public void doGet(HttpServletRequest request, HttpServletResponse response)
      throws IOException, ServletException
   {
      OutputStream out = response.getOutputStream();
      try {
         String command = request.getParameter("_command");

         ServletContext sc = getServletContext();
         String db_url = DB_URL_P1 + sc.getRealPath(DB_URL_P2);
         String template_file_path = sc.getRealPath(TEMPLATE_FILE_PATH_PART);
         Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
         if(command.equals("ColumnChart")) {
            setResponseHeader(response, "ColumnChart.xls");
            CreateColumnChart(out);
         }else if(command.equals("LineChart")){
            setResponseHeader(response, "LineChart.xls");
            CreateLineChart(out);
         }else if(command.equals("StockChart")){
            setResponseHeader(response, "StockChart.xls");
            CreateStockChart(out);
         }else if(command.equals("PivotTable")){
            setResponseHeader(response, "PivotTable.xls");
            CreatePivotTable(out);
         }
      } catch(Exception x) {
         x.printStackTrace();

         // say sorry
         response.setContentType("text/html");
         PrintWriter writer = new PrintWriter(out);

         writer.println("<html>");
         writer.println("<head>");
         writer.println("<title>sorry</title>");
         writer.println("</head>");
         writer.println("<body>");
         writer.println("<h1>Sorry, the demo programme failed for some reason.</h1>");
         x.printStackTrace(writer);
         writer.println("</body>");
         writer.println("</html>");
         writer.close();
      }
   }


   public void doPost(HttpServletRequest request, HttpServletResponse response)
         throws IOException, ServletException
   {
      doGet(request, response);
   }

   private void setResponseHeader(HttpServletResponse response, String resultXLS)
   {
      response.setContentType("application/vnd.ms-excel");
      //response.addHeader("content-disposition", "attachment;  filename=" + resultXLS);
      response.addHeader("content-disposition", "inline;  filename=" + resultXLS);
   }

   private static void CreateColumnChart(OutputStream out) throws IOException
   {
      Workbook workbook = new Workbook();
      Worksheets worksheets = workbook.getWorksheets();
      Worksheet sheet = worksheets.getSheet(0);
      Cells cells = sheet.getCells();
      Cell cell = cells.getCell("B4");
      cell.setValue("January");
      cell = cells.getCell("C4");
      cell.setValue("February");
      //soure data
      cell = cells.getCell("B5");
      cell.setValue(9300);
      cell = cells.getCell("C5");
      cell.setValue(1280);
      cell = cells.getCell("B6");
      cell.setValue(11300);
      cell = cells.getCell("C6");
      cell.setValue(21000);
      cell = cells.getCell("B7");
      cell.setValue(13200);
      cell = cells.getCell("C7");
      cell.setValue(11000);
      Style style = workbook.createStyle();
      style.setNumber(5);
      cells.setRangeStyle(4,6,1,2,style);
      //catergory
      cell = cells.getCell("A5");
      cell.setValue("Chicago");
      cell = cells.getCell("A6");
      cell.setValue("Dallas");
      cell = cells.getCell("A7");
      cell.setValue("Boston");

      //
      //sheet = worksheets.addSheet(SheetType.CHART,"chartsheet")
      Charts charts = sheet.getCharts();

      //Adding a chart to the worksheet
      Chart chart = charts.addChart(ChartType.COLUMN_CLUSTERED,10,5,22,12);
      //Adding NSeries (chart data source) to the chart ranging from "B5" cell to "C7"
      NSeries nSeries = chart.getNSeries();
      nSeries.add("B5:C7",true);

      //Setting the data source for the category data of NSeries
      nSeries.setCategoryData("A5:A7");
      //setting sereis name
      ASeries aSeries = nSeries.get(0);
      aSeries.setName("=B4");
      aSeries = nSeries.get(1);
      aSeries.setName("=C4");

      workbook.save(out);
   }


   private static void CreateStockChart(OutputStream out) throws IOException
   {
      Workbook workbook = new Workbook();
      Worksheets worksheets = workbook.getWorksheets();
      Worksheet sheet = worksheets.getSheet(0);
      Cells cells = sheet.getCells();
      Cell cell = cells.getCell("B1");
      cell.setValue("Volumne");
      cell = cells.getCell("C1");
      cell.setValue("High");
      cell = cells.getCell("D1");
      cell.setValue("Low");
      cell = cells.getCell("E1");
      cell.setValue("Close");
      //soure data
      cell = cells.getCell("B2");
      cell.setValue(41000);
      cell = cells.getCell("C2");
      cell.setValue(27.20);
      cell = cells.getCell("D2");
      cell.setValue(23.49);
      cell = cells.getCell("E2");
      cell.setValue(25.45);
      cell = cells.getCell("B3");
      cell.setValue(35000);
      cell = cells.getCell("C3");
      cell.setValue(25.03);
      cell = cells.getCell("D3");
      cell.setValue(19.55);
      cell = cells.getCell("E3");
      cell.setValue(23.05);
      cell = cells.getCell("B4");
      cell.setValue(30000);
      cell = cells.getCell("C4");
      cell.setValue(19.05);
      cell = cells.getCell("D4");
      cell.setValue(15.12);
      cell = cells.getCell("E4");
      cell.setValue(17.32);
      Style style = workbook.createStyle();
      style.setNumber(7);
      cells.setRangeStyle(1,3,1,4,style);
      //catergory
      cell = cells.getCell("A2");
      cell.setValue("Aspose");
      cell = cells.getCell("A3");
      cell.setValue("Aspose.Cells");
      cell = cells.getCell("A4");
      cell.setValue("Aspose.Words");

      //
      Charts charts = sheet.getCharts();

      //Adding a chart to the worksheet
      Chart chart = charts.addChart(ChartType.STOCK_VOLUME_HIGH_LOW_CLOSE,10,5,22,15);
      //Adding NSeries (chart data source) to the chart ranging from "B5" cell to "C7"
      NSeries nSeries = chart.getNSeries();
      nSeries.add("B2:E4",true);

      //Setting the data source for the category data of NSeries
      nSeries.setCategoryData("A2:A4");
      //setting sereis name
      ASeries aSeries = nSeries.get(0);
      aSeries.setName("=B1");
      aSeries = nSeries.get(1);
      aSeries.setName("=C1");
      aSeries = nSeries.get(2);
      aSeries.setName("=D1");
      aSeries = nSeries.get(3);
      aSeries.setName("=E1");


      workbook.save(out);
   }

   private static void CreateLineChart(OutputStream out) throws IOException
   {
      Workbook workbook = new Workbook();
      Worksheets worksheets = workbook.getWorksheets();
      int iNewSheets = 0;
      // Get rid of all sheets except first one
      int i1 = worksheets.size();
      for(int i = worksheets.size() - 1; i > 0; --i) {
         worksheets.removeSheet(i);
      }
      /* Workbook has a 32000 data point limit on 2D charts
      ** and a 4000 data point on 3D charts */
      int iMaxRows = 32000;

      Worksheet linechart = worksheets.addSheet( SheetType.CHART, "CPUUtil");
      iNewSheets++;
      Worksheet sheet = worksheets.addSheet(SheetType.WORKSHEET, "CPUData");
      //setColumnWidth(sheet);
      iNewSheets++;
      Cells cells = sheet.getCells();
      Cell cell = cells.getCell("B4");
      cell.setValue("saturn");
      cell = cells.getCell("C4");
      cell.setValue("clsol1");
      cell = cells.getCell("D4");
      cell.setValue("clsol2");
      cell = cells.getCell("E4");
      cell.setValue("clsol3");
      cell = cells.getCell("F4");
      cell.setValue("clsol4");
      cell = cells.getCell("G4");
      cell.setValue("tqclss1");
      //source data
      //category
      Calendar calendar;
      double iChicago;
      double iDallas;
      double iBoston;
      double iLA;
      double iSeattle;
      double iCL;
      int iNumDays = 7;
      int iNumHours = 24;
      int iTotRows = iNumDays * iNumHours;
      Random generator = new Random();

      for (int i=1; i < iNumDays + 1; i++) {
        for (int j=0; j < iNumHours; j++ ){
          // Set date
          calendar = new GregorianCalendar(1990, 0, i, j, 0, 0);
          calendar.setTimeZone(TimeZone.getTimeZone("GMT"));
          cell = cells.getCell(((i-1) * iNumHours) + j + 4, 0);
          cell.setValue(calendar);

          // Set Chicago numbers
          iChicago = generator.nextInt(2000)/10000.0;
          iChicago += .80; // Never lower than 80%
          cell = cells.getCell(((i-1) * iNumHours) + j + 4, 1);
          cell.setValue(iChicago);

          // Set Dallas numbers
          iDallas = generator.nextInt(2000)/10000.0;
          iDallas += .60; // Never lower than 60% or higher than 80
          cell = cells.getCell(((i-1) * iNumHours) + j + 4, 2);
          cell.setValue(iDallas);

          // Set Boston numbers
          iBoston = generator.nextInt(500)/10000.0;
          iBoston += .65; // Never lower than 65% or higher than 70
          cell = cells.getCell(((i-1) * iNumHours) + j + 4, 3);
          cell.setValue(iBoston);

          // Set LA numbers
          iLA = generator.nextInt(1000)/10000.0;
          iLA += .30; // Never lower than 30% or higher than 40
          cell = cells.getCell(((i-1) * iNumHours) + j + 4, 4);
          cell.setValue(iLA);

          // Set Seattle numbers
          iSeattle = generator.nextInt(700)/10000.0;
          iSeattle += .72; // Never lower than 72% or higher than 79
          cell = cells.getCell(((i-1) * iNumHours) + j + 4, 5);
          cell.setValue(iSeattle);

          // Set CL numbers
          iCL = generator.nextInt(300)/10000.0;
          iCL += .94; // Never lower than 94% or higher than 97
          cell = cells.getCell(((i-1) * iNumHours) + j + 4, 6);
          cell.setValue(iCL);
        }
      }
      // Set Date format
      Style dateStyle = workbook.createStyle();
      dateStyle.setNumber(22);
      cells.setRangeStyle(4,(4+iTotRows),0,0,dateStyle);
      // Set Data format
      Style style = workbook.createStyle();
      style.setNumber(10);
      cells.setRangeStyle(4,(4+iTotRows),1,6,style);

      /*
      ***************************************************************
      ** Setup the chart
      */
      Charts charts = linechart.getCharts();

      //Adding a chart to the worksheet
      Chart chart = charts.addChart(ChartType.LINE_WITH_DATA_MARKERS,0,0,42,20);

      //Set the title
      Title title = chart.getTitle();
      title.setText("Work Server %busy");

      //Adding NSeries (chart data source) to the chart ranging from "B5" cell to "C7"

      StringBuffer builder = new StringBuffer("CPUData!B5:G");

      // Reset total number of rows to chart for now
      if (iTotRows > iMaxRows)
        iTotRows = iMaxRows;

      builder.append(iTotRows + 4);
      NSeries nSeries = chart.getNSeries();
      nSeries.add(builder.toString(),true);

      //Setting the data source for the category data of NSeries
      StringBuffer builder2 = new StringBuffer("CPUData!A5:A");
      builder2.append(iTotRows + 4);

      nSeries.setCategoryData(builder2.toString());

      CategoryAxis catAxis = chart.getCategoryAxis();
      catAxis.setCategoryType(CategoryType.CATEGORY_SCALE);
      catAxis.setNumber(22);

      //Set the Value Axis title
      Title cTitle = catAxis.getTitle();
      cTitle.setText("Time");
      cTitle.setTextHorizontalAlignment(TextAlignmentType.CENTER);
      catAxis.getMajorGridLines().setVisible(true);
      catAxis.setTickLabelSpacing(12);

      ValueAxis valAxis = chart.getValueAxis();
      valAxis.setMaxValue(1.0);

      //Set the Value Axis title
      Title vTitle = valAxis.getTitle();
      vTitle.setText("%busy");
      vTitle.setRotation(90);

      //setting series name
      ASeries aSeries = nSeries.get(0);
      aSeries.setName("=CPUData!B4");

      aSeries = nSeries.get(1);
      aSeries.setName("=CPUData!C4");

      aSeries = nSeries.get(2);
      aSeries.setName("=CPUData!D4");

      aSeries = nSeries.get(3);
      aSeries.setName("=CPUData!E4");

      aSeries = nSeries.get(4);
      aSeries.setName("=CPUData!F4");

      aSeries = nSeries.get(5);
      aSeries.setName("=CPUData!G4");

      // Remove any leading sheets before the ones we created.
      for(int i = worksheets.size() - (iNewSheets + 1); i >= 0; --i) {
         worksheets.removeSheet(i);
      }

      // Add text box
      PlotArea plotArea = chart.getPlotArea();
      int iPlotX = plotArea.getX();
      int iPlotY = plotArea.getY();
      int iPlotWidth = plotArea.getWidth();
      int iPlotHeight = plotArea.getHeight();
      Shapes shapes = chart.getShapes();

      TextBox textBox = shapes.addTextBoxInChart(iPlotX + (iPlotWidth / 4),iPlotY + (iPlotHeight / 4),iPlotWidth / 2 ,iPlotHeight / 2);
      textBox.setFilled(false);

      textBox.setContent("Line chart demo!!");
      Font boxFont = new Font();
      boxFont.setSize(18);
      boxFont.setName("Arial");

      textBox.setFont(boxFont);
      ShapeLine line = textBox.getLine();
      line.setVisible(false);

      textBox.setTextHorizontalAlignment(TextAlignmentType.CENTER);
      textBox.setTextVerticalAlignment(TextAlignmentType.CENTER);

      workbook.save(out);
   }


   private static void CreatePivotTable(OutputStream out) throws IOException
   {
      Workbook workbook = new Workbook();
      Worksheet sheet = workbook.getWorksheets().getSheet(0);
      Cells cells = sheet.getCells();
      Cell cell = cells.getCell("A1");
      cell.setValue("Sport");
      cell = cells.getCell("B1");
      cell.setValue("Quarter");
      cell = cells.getCell("C1");
      cell.setValue("Sales");


      cell = cells.getCell("A2");
      cell.setValue("Golf");
      cell = cells.getCell("A3");
      cell.setValue("Golf");
      cell = cells.getCell("A4");
      cell.setValue("Tennis");
      cell = cells.getCell("A5");
      cell.setValue("Tennis");
      cell = cells.getCell("A6");
      cell.setValue("Tennis");
      cell = cells.getCell("A7");
      cell.setValue("Tennis");
      cell = cells.getCell("A8");
      cell.setValue("Golf");

      cell = cells.getCell("B2");
      cell.setValue("Qtr3");
      cell = cells.getCell("B3");
      cell.setValue("Qtr4");
      cell = cells.getCell("B4");
      cell.setValue("Qtr3");
      cell = cells.getCell("B5");
      cell.setValue("Qtr4");
      cell = cells.getCell("B6");
      cell.setValue("Qtr3");
      cell = cells.getCell("B7");
      cell.setValue("Qtr4");
      cell = cells.getCell("B8");
      cell.setValue("Qtr3");

      cell = cells.getCell("C2");
      cell.setValue(1500);
      cell = cells.getCell("C3");
      cell.setValue(2000);
      cell = cells.getCell("C4");
      cell.setValue(600);
      cell = cells.getCell("C5");
      cell.setValue(1500);
      cell = cells.getCell("C6");
      cell.setValue(4070);
      cell = cells.getCell("C7");
      cell.setValue(5000);
      cell = cells.getCell("C8");
      cell.setValue(6430);

      PivotTables pivotTables = sheet.getPivotTables();
      int index = pivotTables.add("=A1:C8","E3","PivotTable1");
      PivotTable pivotTable = pivotTables.get(index);
      pivotTable.addFieldToArea(PivotFieldType.ROW,0);
      pivotTable.addFieldToArea(PivotFieldType.COLUMN,1);
      pivotTable.addFieldToArea(PivotFieldType.DATA,2);
      pivotTable.setSaveData(true);
//      workbook.open("F:\\FileTemp\\book2.xls");
//      Worksheet sheet = workbook.getWorksheets().getSheet(0);
//      Cells cells = sheet.getCells();
//      Cell cell = cells.getCell("A21");
//      cell.setValue("footerball");
      workbook.save(out);
   }
}
