package PageSetup;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.aspose.cells.OrderType;
import com.aspose.cells.PageSetup;
import com.aspose.cells.PrintCommentsType;
import com.aspose.cells.Workbook;
import com.aspose.cells.Worksheet;
import com.aspose.cells.Worksheets;

public class SettingPrintOptions extends HttpServlet {
  private static final String TEMPLATE_FILE_PATH_PART = "/WEB-INF/Designer/book1.xls";

public void doGet(HttpServletRequest request, HttpServletResponse response)
throws IOException, ServletException
{
OutputStream out = response.getOutputStream();
try
{
ServletContext sc = getServletContext();
String template_file_path = sc.getRealPath(TEMPLATE_FILE_PATH_PART);

setResponseHeader(response, "SettingPrintOptions.xls");
CreateStaticReport(out, template_file_path);
}
catch (Exception ex)
{
ex.printStackTrace();

// say sorry
response.setContentType("text/html");
PrintWriter writer = new PrintWriter(out);

writer.println("<html>");
writer.println("<head>");
writer.println("<title>sorry</title>");
writer.println("</head>");
writer.println("<body>");
writer.println("<h1>Sorry, the demo programme failed for some reason.</h1>");
ex.printStackTrace(writer);
writer.println("</body>");
writer.println("</html>");
writer.close();
}
}

public void doPost(HttpServletRequest request, HttpServletResponse response)
throws IOException, ServletException
{
doGet(request, response);
}

private void setResponseHeader(HttpServletResponse response, String resultXLS)
{
response.setContentType("application/vnd.ms-excel");
response.addHeader("content-disposition", "inline;  filename=" + resultXLS);
}

private void CreateStaticReport(OutputStream out, String template_file_path) throws IOException
{
Workbook workbook = new Workbook();

workbook.open(template_file_path);

Worksheets worksheets = workbook.getWorksheets();
Worksheet worksheet = worksheets.getSheet(0);

PageSetup pageSetup = worksheet.getPageSetup();
pageSetup.setPrintArea("A1:G5");
pageSetup.setPrintTitleColumns("$A:$B");
pageSetup.setPrintTitleRows("$1:$2");
//Allow to print gridlines
pageSetup.setPrintGridlines(true);
//Allow to print row/column headings
pageSetup.setPrintHeadings(true);
//Allow to print worksheet in black & white mode
pageSetup.setBlackAndWhite(true);
//Allow to print comments as displayed on worksheet
pageSetup.setPrintComments(PrintCommentsType.PRINT_IN_PLACE);
//Allow to print worksheet with draft quality
pageSetup.setDraftQuality(true);
//Allow to print cell errors
pageSetup.setPageOrder(OrderType.DOWN_THEN_OVER);

//Saving the Excel file
workbook.save(out);
}

}
