package chart;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.aspose.cells.CategoryAxis;
import com.aspose.cells.Chart;
import com.aspose.cells.ChartType;
import com.aspose.cells.Charts;
import com.aspose.cells.Color;
import com.aspose.cells.Font;
import com.aspose.cells.Legend;
import com.aspose.cells.LegendPositionType;
import com.aspose.cells.NSeries;
import com.aspose.cells.Title;
import com.aspose.cells.ValueAxis;
import com.aspose.cells.Workbook;
import com.aspose.cells.Worksheet;
import com.aspose.cells.Worksheets;

public class ClusteredColumnServlet  extends HttpServlet {
  public ClusteredColumnServlet()  {
  }

  private static final String TEMPLATE_FILE_PATH_PART = "/WEB-INF/Designer/Chart/Template.xls";

  public void doGet(HttpServletRequest request, HttpServletResponse response)
     throws IOException, ServletException
  {
    OutputStream out = response.getOutputStream();
    try
    {
        ServletContext sc = getServletContext();
        String template_file_path = sc.getRealPath(TEMPLATE_FILE_PATH_PART);

        int gapWidth = Integer.parseInt(request.getParameter("GapWidth"));
        String categoryAxisTitle = request.getParameter("CategoryAxisTitle");
        String valueAxisTitle = request.getParameter("ValueAxisTitle");
        Double maxValue = Double.parseDouble(request.getParameter("ValueMaxValue"));
        Double minValue = Double.parseDouble(request.getParameter("ValueMinValue"));
        Double majorUnit = Double.parseDouble(request.getParameter("ValueMajorUnit"));
        Double minorUnit = Double.parseDouble(request.getParameter("ValueMinorUnit"));

        setResponseHeader(response, "ClusteredColumn.xls");
        CreateColumnChart(out, template_file_path, gapWidth, categoryAxisTitle,
                          valueAxisTitle, majorUnit, maxValue, minorUnit, minValue);
     }
    catch (Exception ex)
    {
        ex.printStackTrace();

        // say sorry
        response.setContentType("text/html");
        PrintWriter writer = new PrintWriter(out);

        writer.println("<html>");
        writer.println("<head>");
        writer.println("<title>sorry</title>");
        writer.println("</head>");
        writer.println("<body>");
        writer.println("<h1>Sorry, the demo programme failed for some reason.</h1>");
        ex.printStackTrace(writer);
        writer.println("</body>");
        writer.println("</html>");
        writer.close();
    }
  }

  public void doPost(HttpServletRequest request, HttpServletResponse response)
         throws IOException, ServletException
   {
      doGet(request, response);
   }

  private void setResponseHeader(HttpServletResponse response, String resultXLS)
  {
     response.setContentType("application/vnd.ms-excel");
     response.addHeader("content-disposition", "inline;  filename=" + resultXLS);
  }

  private void CreateColumnChart(OutputStream out, String template_file_path, int gapWidth, String categoryAxisTitle, String valueAxisTitle,
                                 Double majorUnit, Double maxValue, Double minorUnit, Double minValue) throws IOException
 {
    Workbook workbook = new Workbook();
    workbook.open(template_file_path);
    CreateStaticReport(workbook, gapWidth, categoryAxisTitle, valueAxisTitle, majorUnit,
        maxValue, minorUnit, minValue);
    workbook.save(out);
 }

 private void CreateStaticReport(Workbook workbook, int gapWidth, String categoryAxisTitle, String valueAxisTitle,
                                 Double majorUnit, Double maxValue, Double minorUnit, Double minValue)
 {
   Worksheets worksheets = workbook.getWorksheets();
   Worksheet worksheet = worksheets.getSheet(0);
   //Set the name of the worksheet
   worksheet.setName("Clustered Column");

   //Create chart
   Charts charts = worksheet.getCharts();
   Chart chart = charts.addChart(ChartType.COLUMN_CLUSTERED, 5, 1, 29, 10);

   NSeries nSeries = chart.getNSeries();

   //Add the nseries collection to a chart
   nSeries.add("B2:B4", true);
   //Set the range of category axis values
   nSeries.setCategoryData("A2:A4");

   for (int i = 0; i < nSeries.size(); i++)
   {
     nSeries.get(i).getDataLabels().setValueShown(true);
     nSeries.get(i).setColorVaried(true);
   }

   Legend legend = chart.getLegend();
   //Set the legend position type
   legend.setPosition(LegendPositionType.TOP);
   chart.setGapWidth(gapWidth);

   //Set properties of chart title
   Title title = chart.getTitle();
   title.setText("Marketing Costs by Region");
   Font font1 = new Font();
   font1.setBold(true);
   font1.setColor(Color.BLACK);
   font1.setSize(12);
   title.setFont(font1);

   //Set properties of categoryaxis title
   CategoryAxis categoryAxis = chart.getCategoryAxis();
   categoryAxis.getTitle().setText(categoryAxisTitle);
   Font font2 = new Font();
   font2.setName("Arial");
   font2.setBold(true);
   font2.setColor(Color.BLACK);
   font2.setSize(10);
   categoryAxis.getTitle().setFont(font2);

   //Set properties of valueaxis title
   ValueAxis valueAxis = chart.getValueAxis();
   valueAxis.getTitle().setText(valueAxisTitle);
   valueAxis.getTitle().setFont(font2);
   valueAxis.getTitle().setRotation(90);
   valueAxis.setMajorUnit(majorUnit);
   valueAxis.setMaxValue(maxValue);
   valueAxis.setMinorUnit(minorUnit);
   valueAxis.setMinValue(minValue);
   valueAxis.getMajorGridLines().setVisible(false);
 }
 }
