package chart;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.aspose.cells.Chart;
import com.aspose.cells.ChartType;
import com.aspose.cells.Charts;
import com.aspose.cells.Color;
import com.aspose.cells.Font;
import com.aspose.cells.Legend;
import com.aspose.cells.LegendPositionType;
import com.aspose.cells.NSeries;
import com.aspose.cells.Title;
import com.aspose.cells.Workbook;
import com.aspose.cells.Worksheet;
import com.aspose.cells.Worksheets;

public class ExplodedPieServlet extends HttpServlet {

  private static final String TEMPLATE_FILE_PATH_PART = "/WEB-INF/Designer/Chart/PieTemplate.xls";

   public void doGet(HttpServletRequest request, HttpServletResponse response)
  throws IOException, ServletException
{
 OutputStream out = response.getOutputStream();
 try
 {
     ServletContext sc = getServletContext();
     String template_file_path = sc.getRealPath(TEMPLATE_FILE_PATH_PART);

     String parameter = request.getParameter("CheckBoxShow3D");
     setResponseHeader(response, "ExplodedPie.xls");
     CreatePieChart(out, template_file_path, parameter);
  }
 catch (Exception ex)
 {
     ex.printStackTrace();

     // say sorry
     response.setContentType("text/html");
     PrintWriter writer = new PrintWriter(out);

     writer.println("<html>");
     writer.println("<head>");
     writer.println("<title>sorry</title>");
     writer.println("</head>");
     writer.println("<body>");
     writer.println("<h1>Sorry, the demo programme failed for some reason.</h1>");
     ex.printStackTrace(writer);
     writer.println("</body>");
     writer.println("</html>");
     writer.close();
 }
}

public void doPost(HttpServletRequest request, HttpServletResponse response)
      throws IOException, ServletException
{
   doGet(request, response);
}

private void setResponseHeader(HttpServletResponse response, String resultXLS)
{
  response.setContentType("application/vnd.ms-excel");
  response.addHeader("content-disposition", "inline;  filename=" + resultXLS);
}

private void CreatePieChart(OutputStream out, String template_file_path, String parameter) throws IOException
{
 Workbook workbook = new Workbook();

  workbook.open(template_file_path);

  CreateStaticReport(workbook, parameter);

  workbook.save(out);
}

private void CreateStaticReport(Workbook workbook, String parameter)
{
  Worksheets worksheets = workbook.getWorksheets();
  Worksheet dataSheet = worksheets.getSheet(0);
  Worksheet worksheet = worksheets.addSheet();
  //Set the name of worksheet
  dataSheet.setName("Data");
  worksheet.setName("Chart");

  //Create chart
  Charts charts = worksheet.getCharts();
  Chart chart = null;
  if (parameter != null)
    chart = charts.addChart(ChartType.PIE_3D_EXPLODED, 1, 1, 25, 10);
  else
    chart = charts.addChart(ChartType.PIE_EXPLODED, 1, 1, 25, 10);
  

  //Set properties of nseries
  NSeries nSeries = chart.getNSeries();
  nSeries.add("Data!B2:B8", true);
  nSeries.setCategoryData("Data!A2:A8");
  for (int i = 0; i < nSeries.size(); i ++)
  {
    nSeries.get(i).setColorVaried(true);
  }

  //Set properties of chart title
  Title title = chart.getTitle();
  title.setText("Sales By Region");
  Font font = new Font();
  font.setBold(true);
  font.setColor(Color.BLACK);
  font.setSize(12);
  title.setFont(font);

  //Set the legend position type
  Legend legend = chart.getLegend();
  legend.setPosition(LegendPositionType.RIGHT);
}
}
