package chart;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.aspose.cells.CategoryAxis;
import com.aspose.cells.Cells;
import com.aspose.cells.Chart;
import com.aspose.cells.ChartType;
import com.aspose.cells.Charts;
import com.aspose.cells.Color;
import com.aspose.cells.Font;
import com.aspose.cells.Legend;
import com.aspose.cells.LegendPositionType;
import com.aspose.cells.NSeries;
import com.aspose.cells.TickMarkerType;
import com.aspose.cells.Title;
import com.aspose.cells.ValueAxis;
import com.aspose.cells.Workbook;
import com.aspose.cells.Worksheet;
import com.aspose.cells.Worksheets;

public class Line3DServlet extends HttpServlet {
  public Line3DServlet() {
    try {
      jbInit();
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  private static final String TEMPLATE_FILE_PATH_PART = "/WEB-INF/Designer/Chart/AreaTemplate.xls";

  public void doGet(HttpServletRequest request, HttpServletResponse response)
   throws IOException, ServletException
{
  OutputStream out = response.getOutputStream();
  try
  {
      ServletContext sc = getServletContext();
      String template_file_path = sc.getRealPath(TEMPLATE_FILE_PATH_PART);

      int cLabelsRotation = Integer.parseInt(request.getParameter("CLabelsRotation"));
      int vLabelsRotation =Integer.parseInt(request.getParameter("VLabelsRotation"));
      int majorTickMarkType = Integer.parseInt(request.getParameter("MajorTickMarkType"));
      int minorTickMarkType = Integer.parseInt(request.getParameter("MinorTickMarkType"));
      setResponseHeader(response, "Line3D.xls");
      CreateLineChart(out, template_file_path, cLabelsRotation, vLabelsRotation,
          majorTickMarkType, minorTickMarkType);
   }
  catch (Exception ex)
  {
      ex.printStackTrace();

      // say sorry
      response.setContentType("text/html");
      PrintWriter writer = new PrintWriter(out);

      writer.println("<html>");
      writer.println("<head>");
      writer.println("<title>sorry</title>");
      writer.println("</head>");
      writer.println("<body>");
      writer.println("<h1>Sorry, the demo programme failed for some reason.</h1>");
      ex.printStackTrace(writer);
      writer.println("</body>");
      writer.println("</html>");
      writer.close();
  }
}

public void doPost(HttpServletRequest request, HttpServletResponse response)
       throws IOException, ServletException
 {
    doGet(request, response);
 }

private void setResponseHeader(HttpServletResponse response, String resultXLS)
{
   response.setContentType("application/vnd.ms-excel");
   response.addHeader("content-disposition", "inline;  filename=" + resultXLS);
}

 private void CreateLineChart(OutputStream out, String template_file_path, int cLabelsRotation, int vLabelsRotation,
                                 int majorTickMarkType, int minorTickMarkType) throws IOException
{
  Workbook workbook = new Workbook();
   workbook.open(template_file_path);
   CreateStaticReport(workbook, cLabelsRotation, vLabelsRotation, majorTickMarkType,
                      minorTickMarkType);
   workbook.save(out);
}

 private void CreateStaticReport(Workbook workbook, int cLabelsRotation, int vLabelsRotation,
                                 int majorTickMarkType, int minorTickMarkType)
 {
   Worksheets worksheets = workbook.getWorksheets();
   Worksheet worksheet = worksheets.getSheet(0);
   //Set the name of worksheet
   worksheet.setName("3D Line");

   //Create chart
   Charts charts = worksheet.getCharts();
   Chart chart = charts.addChart(ChartType.LINE_3D, 5, 1, 29, 10);

   //Set properties of nseries
   NSeries nSeries = chart.getNSeries();
   nSeries.add("B2:F4", false);
   nSeries.setCategoryData("B1:F1");

   Cells cells = worksheet.getCells();
   String temp = "";
   for (int i = 0; i < nSeries.size(); i ++) {
     //Set the name of nseries
     temp = cells.getCell(i + 1, 0).getStringValue();
     nSeries.get(i).setName(temp);
   }

   //Set properties of chart
   chart.setRotation(30);

   //Set properties of chart title
   Title title = chart.getTitle();
   title.setText("Sales By Region");
   Font font1 = new Font();
   font1.setBold(true);
   font1.setColor(Color.BLACK);
   font1.setSize(12);
   title.setFont(font1);

   //Set properties of categoryaxis title
   CategoryAxis categoryAxis = chart.getCategoryAxis();
   categoryAxis.getTitle().setText("Year(2002-2006)");
   Font font2 = new Font();
   font2.setBold(true);
   font2.setColor(Color.BLACK);
   font2.setSize(10);
   categoryAxis.getTitle().setFont(font2);
   categoryAxis.setRotation(cLabelsRotation);

   //Set properies to valueaxis
   ValueAxis valueAxis = chart.getValueAxis();
   valueAxis.setRotation(vLabelsRotation);
   switch (majorTickMarkType)
   {
      case 0:
        valueAxis.setMajorTickMark(TickMarkerType.NONE);
        break;
      case 1:
        valueAxis.setMajorTickMark(TickMarkerType.INSIDE);
        break;
      case 2:
        valueAxis.setMajorTickMark(TickMarkerType.OUTSIDE);
        break;
      case 3:
        valueAxis.setMajorTickMark(TickMarkerType.CROSS);
        break;
   }
   switch (minorTickMarkType)
   {
     case 0:
        valueAxis.setMinorTickMark(TickMarkerType.NONE);
        break;
      case 1:
        valueAxis.setMinorTickMark(TickMarkerType.INSIDE);
        break;
      case 2:
        valueAxis.setMinorTickMark(TickMarkerType.OUTSIDE);
        break;
      case 3:
        valueAxis.setMinorTickMark(TickMarkerType.CROSS);
        break;
   }

 //Set the position of legend
 Legend legend = chart.getLegend();
 legend.setPosition(LegendPositionType.TOP);
}

  private void jbInit() throws Exception {
  }
}
