package chart;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.aspose.cells.Cells;
import com.aspose.cells.Chart;
import com.aspose.cells.ChartType;
import com.aspose.cells.Charts;
import com.aspose.cells.Color;
import com.aspose.cells.Font;
import com.aspose.cells.NSeries;
import com.aspose.cells.SheetType;
import com.aspose.cells.Title;
import com.aspose.cells.ValueAxis;
import com.aspose.cells.Workbook;
import com.aspose.cells.Worksheet;
import com.aspose.cells.Worksheets;

public class PercentStackedBarServlet extends HttpServlet {
  private static final String TEMPLATE_FILE_PATH_PART = "/WEB-INF/Designer/Chart/PercentStacked.xls";

  public void doGet(HttpServletRequest request, HttpServletResponse response)
     throws IOException, ServletException
  {
    OutputStream out = response.getOutputStream();
    try
    {
        ServletContext sc = getServletContext();
        String template_file_path = sc.getRealPath(TEMPLATE_FILE_PATH_PART);

        String command = request.getParameter("CheckBoxShow3D");
        setResponseHeader(response, "PercentStackedBar.xls");
        CreateBarChart(out, template_file_path, command);
     }
    catch (Exception ex)
    {
        ex.printStackTrace();

        // say sorry
        response.setContentType("text/html");
        PrintWriter writer = new PrintWriter(out);

        writer.println("<html>");
        writer.println("<head>");
        writer.println("<title>sorry</title>");
        writer.println("</head>");
        writer.println("<body>");
        writer.println("<h1>Sorry, the demo programme failed for some reason.</h1>");
        ex.printStackTrace(writer);
        writer.println("</body>");
        writer.println("</html>");
        writer.close();
    }
  }

  public void doPost(HttpServletRequest request, HttpServletResponse response)
         throws IOException, ServletException
   {
      doGet(request, response);
   }

  private void setResponseHeader(HttpServletResponse response, String resultXLS)
  {
     response.setContentType("application/vnd.ms-excel");
     response.addHeader("content-disposition", "inline;  filename=" + resultXLS);
  }

  private void CreateBarChart(OutputStream out, String template_file_path, String Parameter) throws IOException
  {
     Workbook workbook = new Workbook();

     workbook.open(template_file_path);

     CreateStaticReport(workbook, Parameter);

     workbook.save(out);
  }

  private void CreateStaticReport(Workbook workbook, String Parameter)
  {
    Worksheets worksheets = workbook.getWorksheets();
    Worksheet worksheet = worksheets.addSheet(SheetType.CHART);
    Worksheet dataSheet = worksheets.getSheet(0);
    //Set the name of worksheet
    dataSheet.setName("Data");
    worksheet.setName("Chart");

    //Create Chart
    Charts charts = worksheet.getCharts();
    Chart chart = null;
    if (Parameter != null)
      chart = charts.addChart(ChartType.BAR_3D_100PERCENT_STACKED, 0, 0, 0, 0);
    else
      chart = charts.addChart(ChartType.BAR_100PERCENT_STACKED, 0, 0, 0, 0);

    //Set properties of nseries
    NSeries nSeries = chart.getNSeries();
    nSeries.add("Data!B2:E4", false);
    nSeries.setCategoryData("Data!B1:E1");

    Cells cells = dataSheet.getCells();
    String temp = "";
    for (int i = 0; i < nSeries.size(); i ++)
    {
      temp = cells.getCell(i + 1, 0).getStringValue();
      nSeries.get(i).setName(temp);
    }

    //Set properties of chart title
    Title title = chart.getTitle();
    title.setText("Product contribution to total sales");
    Font font1 = new Font();
    font1.setBold(true);
    font1.setColor(Color.BLACK);
    font1.setSize(12);
    title.setFont(font1);

    //Set properties of valueaxis title
    ValueAxis valueAxis = chart.getValueAxis();
    valueAxis.getTitle().setText("% of total sales");
    Font font2 = new Font();
    font2.setColor(Color.BLACK);
    font2.setBold(true);
    font2.setSize(10);
    valueAxis.getTitle().setFont(font2);
  }
}
