package chart;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.aspose.cells.Chart;
import com.aspose.cells.ChartType;
import com.aspose.cells.Charts;
import com.aspose.cells.Color;
import com.aspose.cells.Font;
import com.aspose.cells.NSeries;
import com.aspose.cells.Title;
import com.aspose.cells.Workbook;
import com.aspose.cells.Worksheet;
import com.aspose.cells.Worksheets;

public class PyramidBarServlet extends HttpServlet {

  private static final String TEMPLATE_FILE_PATH_PART = "/WEB-INF/Designer/Chart/Template2.xls";

public void doGet(HttpServletRequest request, HttpServletResponse response)
throws IOException, ServletException
{
OutputStream out = response.getOutputStream();
try
{
  ServletContext sc = getServletContext();
  String template_file_path = sc.getRealPath(TEMPLATE_FILE_PATH_PART);

  setResponseHeader(response, "PyramidBar.xls");
  int chartType = Integer.parseInt(request.getParameter("ChartTypeList"));
  CreatePyramidChart(out, template_file_path, chartType);
}
catch (Exception ex)
{
  ex.printStackTrace();

  // say sorry
  response.setContentType("text/html");
  PrintWriter writer = new PrintWriter(out);

  writer.println("<html>");
  writer.println("<head>");
  writer.println("<title>sorry</title>");
  writer.println("</head>");
  writer.println("<body>");
  writer.println("<h1>Sorry, the demo programme failed for some reason.</h1>");
  ex.printStackTrace(writer);
  writer.println("</body>");
  writer.println("</html>");
  writer.close();
}
}

public void doPost(HttpServletRequest request, HttpServletResponse response)
   throws IOException, ServletException
{
doGet(request, response);
}

private void setResponseHeader(HttpServletResponse response, String resultXLS)
{
response.setContentType("application/vnd.ms-excel");
response.addHeader("content-disposition", "inline;  filename=" + resultXLS);
}

private void CreatePyramidChart(OutputStream out, String template_file_path, int chartType) throws IOException
{
Workbook workbook = new Workbook();

workbook.open(template_file_path);
CreateStaticReport(workbook, chartType);

workbook.save(out);
}

  private void CreateStaticReport(Workbook workbook, int chartType)
  {
    Worksheets worksheets = workbook.getWorksheets();
    Worksheet dataSheet = worksheets.getSheet(0);
    Worksheet worksheet = worksheets.addSheet();
    //Set the name of worksheet
    dataSheet.setName("Data");
    worksheet.setName("Chart");

    //Create chart
    Charts charts = worksheet.getCharts();
    Chart chart = null;
    switch (chartType)
    {
      case 0:
        chart = charts.addChart(ChartType.PYRAMID_BAR, 1, 1, 25, 10);
        break;
      case 1:
        chart = charts.addChart(ChartType.PYRAMID_BAR_STACKED, 1, 1, 25, 10);
        break;
    }

    //Set properties of nseries
    NSeries nSeries = chart.getNSeries();
    nSeries.add("Data!B2:C12",true);
    nSeries.setCategoryData("Data!A2:A12");

    //Set properties of chart
    chart.setElevation(15);
    chart.setRotation(20);
    chart.setLegendShown(false);
    chart.setGapWidth(10);
    chart.setDepthPercent(280);

    //Set properties of chart title
    Title title = chart.getTitle();
    title.setText("Number of Employees");
    Font font = new Font();
    font.setBold(true);
    font.setColor(Color.BLACK);
    font.setSize(12);
    title.setFont(font);
  }
}
