package chart;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.aspose.cells.Cells;
import com.aspose.cells.Chart;
import com.aspose.cells.ChartType;
import com.aspose.cells.Charts;
import com.aspose.cells.Color;
import com.aspose.cells.Font;
import com.aspose.cells.NSeries;
import com.aspose.cells.Title;
import com.aspose.cells.Workbook;
import com.aspose.cells.Worksheet;
import com.aspose.cells.Worksheets;

public class RadarServlet extends HttpServlet {
       private static final String TEMPLATE_FILE_PATH_PART = "/WEB-INF/Designer/Chart/RadarTemplate.xls";

       public void doGet(HttpServletRequest request, HttpServletResponse response)
 throws IOException, ServletException
{
OutputStream out = response.getOutputStream();
try
{
    ServletContext sc = getServletContext();
    String template_file_path = sc.getRealPath(TEMPLATE_FILE_PATH_PART);

    setResponseHeader(response, "Radar.xls");
    int chartType = Integer.parseInt(request.getParameter("ChartTypeList"));
    CreateRadarChart(out, template_file_path, chartType);
 }
catch (Exception ex)
{
    ex.printStackTrace();

    // say sorry
    response.setContentType("text/html");
    PrintWriter writer = new PrintWriter(out);

    writer.println("<html>");
    writer.println("<head>");
    writer.println("<title>sorry</title>");
    writer.println("</head>");
    writer.println("<body>");
    writer.println("<h1>Sorry, the demo programme failed for some reason.</h1>");
    ex.printStackTrace(writer);
    writer.println("</body>");
    writer.println("</html>");
    writer.close();
}
}

public void doPost(HttpServletRequest request, HttpServletResponse response)
     throws IOException, ServletException
{
  doGet(request, response);
}

private void setResponseHeader(HttpServletResponse response, String resultXLS)
{
 response.setContentType("application/vnd.ms-excel");
 response.addHeader("content-disposition", "inline;  filename=" + resultXLS);
}

private void CreateRadarChart(OutputStream out, String template_file_path, int chartType) throws IOException
{
Workbook workbook = new Workbook();

 workbook.open(template_file_path);

 CreateStaticReport(workbook, chartType);

 workbook.save(out);
}

  private void CreateStaticReport(Workbook workbook, int chartType)
  {
    Worksheets worksheets = workbook.getWorksheets();
    Worksheet worksheet = worksheets.getSheet(0);
    //Set the name of worksheet
    worksheet.setName("Radar");

    //Create chart
    Charts charts = worksheet.getCharts();
    Chart chart = null;
    switch (chartType)
    {
      case 0:
        chart = charts.addChart(ChartType.RADAR, 5, 1, 29, 10);
        break;
      case 1:
        chart = charts.addChart(ChartType.RADAR_WITH_DATA_MARKERS, 5, 1, 29, 10);
        break;
    }

    //Set properties of nseries
    NSeries nSeries = chart.getNSeries();
    nSeries.add("B2:G4", false);
    nSeries.setCategoryData("B1:G1");

    Cells cells = worksheet.getCells();
    String temp = "";
    for (int i = 0; i < nSeries.size(); i ++)
    {
      temp = cells.getCell(i + 1, 0).getStringValue();
      nSeries.get(i).setName(temp);
    }

    //Set properties of chart title
    Title title = chart.getTitle();
    title.setText("Nutritional Analysis");
    Font font = new Font();
    font.setBold(true);
    font.setColor(Color.BLACK);
    font.setSize(12);
    title.setFont(font);
  }
}
