package chart;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.aspose.cells.CategoryAxis;
import com.aspose.cells.Chart;
import com.aspose.cells.ChartType;
import com.aspose.cells.Charts;
import com.aspose.cells.Color;
import com.aspose.cells.Font;
import com.aspose.cells.Legend;
import com.aspose.cells.LegendPositionType;
import com.aspose.cells.NSeries;
import com.aspose.cells.Title;
import com.aspose.cells.ValueAxis;
import com.aspose.cells.Workbook;
import com.aspose.cells.Worksheet;
import com.aspose.cells.Worksheets;

public class ScatterConnectedByLinesServlet extends HttpServlet {

  private static final String TEMPLATE_FILE_PATH_PART = "/WEB-INF/Designer/Chart/ScatterTemplate.xls";

  public void doGet(HttpServletRequest request, HttpServletResponse response)
 throws IOException, ServletException
{
OutputStream out = response.getOutputStream();
try
{
    ServletContext sc = getServletContext();
    String template_file_path = sc.getRealPath(TEMPLATE_FILE_PATH_PART);

    setResponseHeader(response, "ScatterConnected.xls");
    int chartType = Integer.parseInt(request.getParameter("ChartTypeList"));
    CreateScatterChart(out, template_file_path, chartType);
 }
catch (Exception ex)
{
    ex.printStackTrace();

    // say sorry
    response.setContentType("text/html");
    PrintWriter writer = new PrintWriter(out);

    writer.println("<html>");
    writer.println("<head>");
    writer.println("<title>sorry</title>");
    writer.println("</head>");
    writer.println("<body>");
    writer.println("<h1>Sorry, the demo programme failed for some reason.</h1>");
    ex.printStackTrace(writer);
    writer.println("</body>");
    writer.println("</html>");
    writer.close();
}
}

public void doPost(HttpServletRequest request, HttpServletResponse response)
     throws IOException, ServletException
{
  doGet(request, response);
}

private void setResponseHeader(HttpServletResponse response, String resultXLS)
{
 response.setContentType("application/vnd.ms-excel");
 response.addHeader("content-disposition", "inline;  filename=" + resultXLS);
}

private void CreateScatterChart(OutputStream out, String template_file_path, int chartType) throws IOException
{
Workbook workbook = new Workbook();

 workbook.open(template_file_path);

 CreateStaticReport(workbook, chartType);

 workbook.save(out);
}

  private void CreateStaticReport(Workbook workbook ,int chartType)
  {
    Worksheets worksheets = workbook.getWorksheets();
    Worksheet dataSheet = worksheets.getSheet(0);
    Worksheet worksheet = worksheets.addSheet();
    //Set the name of worksheet
    dataSheet.setName("Data");
    worksheet.setName("Chart");

    //Create chart
    Charts charts = worksheet.getCharts();
    Chart chart = null;
    switch (chartType)
    {
      case 0:
        chart = charts.addChart(ChartType.SCATTER_CONNECTED_BY_CURVES_WITH_DATA_MARKE, 1, 1, 25, 10);
        break;
      case 1:
        chart = charts.addChart(ChartType.SCATTER_CONNECTED_BY_CURVES_WITHOUT_DATA_MARKE, 1, 1, 25, 10);
        break;
      case 2:
        chart = charts.addChart(ChartType.SCATTER_CONNECTED_BY_LINES_WITH_DATA_MARKE, 1, 1, 25, 10);
        break;
      case 3:
        chart = charts.addChart(ChartType.SCATTER_CONNECTED_BY_LINES_WITHOUT_DATA_MARKE, 1, 1, 25, 10);
        break;
    }

    //Set properties of nseries
    NSeries nSeries = chart.getNSeries();
    nSeries.add("Data!A2:B10", true);
    for (int i = 0; i < nSeries.size(); i ++)
    {
      nSeries.get(i).setName("Particulate");
    }

    //Set properties of chart title
    Title title = chart.getTitle();
    Font font1 = new Font();
    font1.setBold(true);
    font1.setColor(Color.BLACK);
    font1.setSize(12);
    title.setFont(font1);

    //Set properties of categoryaxis title
    CategoryAxis categoryAxis = chart.getCategoryAxis();
    categoryAxis.getTitle().setText("Daily Rainfall");
    Font font2 = new Font();
    font2.setBold(true);
    font2.setColor(Color.BLACK);
    font2.setSize(10);
    categoryAxis.getTitle().setFont(font2);

   //Set properties of valueaxis title
   ValueAxis valueAxis = chart.getValueAxis();
   valueAxis.getTitle().setText("Particulate");
   Font font3 = new Font();
   font3.setBold(true);
   font3.setColor(Color.BLACK);
   font3.setSize(10);
   valueAxis.getTitle().setFont(font3);
   valueAxis.getTitle().setRotation(90);

   //Set the legend position type
   Legend legend = chart.getLegend();
   legend.setPosition(LegendPositionType.TOP);
  }
}

