package chart;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.aspose.cells.CategoryAxis;
import com.aspose.cells.Cells;
import com.aspose.cells.Chart;
import com.aspose.cells.ChartType;
import com.aspose.cells.Charts;
import com.aspose.cells.Color;
import com.aspose.cells.Font;
import com.aspose.cells.NSeries;
import com.aspose.cells.Title;
import com.aspose.cells.ValueAxis;
import com.aspose.cells.Workbook;
import com.aspose.cells.Worksheet;
import com.aspose.cells.Worksheets;

public class Surface3DServlet extends HttpServlet {

  private static final String TEMPLATE_FILE_PATH_PART = "/WEB-INF/Designer/Chart/SurfaceTemplate.xls";

 public void doGet(HttpServletRequest request, HttpServletResponse response)
throws IOException, ServletException
{
OutputStream out = response.getOutputStream();
try
{
   ServletContext sc = getServletContext();
   String template_file_path = sc.getRealPath(TEMPLATE_FILE_PATH_PART);

   setResponseHeader(response, "3DSurface.xls");
   int chartType = Integer.parseInt(request.getParameter("ChartTypeList"));
   CreateSurfaceChart(out, template_file_path, chartType);
}
catch (Exception ex)
{
   ex.printStackTrace();

   // say sorry
   response.setContentType("text/html");
   PrintWriter writer = new PrintWriter(out);

   writer.println("<html>");
   writer.println("<head>");
   writer.println("<title>sorry</title>");
   writer.println("</head>");
   writer.println("<body>");
   writer.println("<h1>Sorry, the demo programme failed for some reason.</h1>");
   ex.printStackTrace(writer);
   writer.println("</body>");
   writer.println("</html>");
   writer.close();
}
}

public void doPost(HttpServletRequest request, HttpServletResponse response)
    throws IOException, ServletException
{
 doGet(request, response);
}

private void setResponseHeader(HttpServletResponse response, String resultXLS)
{
response.setContentType("application/vnd.ms-excel");
response.addHeader("content-disposition", "inline;  filename=" + resultXLS);
}

private void CreateSurfaceChart(OutputStream out, String template_file_path, int chartType) throws IOException
{
Workbook workbook = new Workbook();

workbook.open(template_file_path);

CreateStaticReport(workbook, chartType);

workbook.save(out);
}

  private void CreateStaticReport(Workbook workbook, int chartType)
  {
    Worksheets worksheets = workbook.getWorksheets();
    Worksheet worksheet = worksheets.getSheet(0);
    //Set the name of worksheet
    worksheet.setName("3D Surface");

    //Create chart
    Charts charts = worksheet.getCharts();
    Chart chart = null;
    int chartIndex = 0;
    switch (chartIndex)
    {
      case 0:
        chart = charts.addChart(ChartType.SURFACE_3D, 1, 7, 25, 16);
        break;
      case 1:
        chart = charts.addChart(ChartType.SURFACE_WIREFRAME_3D, 1, 7, 25, 16);
        break;
    }
    

    //Set properties of nseries
    NSeries nSeries = chart.getNSeries();
    nSeries.add("B3:F11", true);
    nSeries.setCategoryData("A3:A11");

    Cells cells = worksheet.getCells();
    String temp = "";
    for (int i = 0; i < nSeries.size(); i ++)
    {
      temp = cells.getCell(0, i + 1).getStringValue();
      nSeries.get(i).setName(temp);
    }

    //Set properties of chart title
    Title title = chart.getTitle();
    title.setText("Tensile strenth Measurements");
    Font font1 = new Font();
    font1.setBold(true);
    font1.setColor(Color.BLACK);
    font1.setSize(12);
    title.setFont(font1);

    //Set properties of categoryaxis title
    CategoryAxis categoryAxis = chart.getCategoryAxis();
    categoryAxis.getTitle().setText("Seconds");
    Font font2 = new Font();
    font2.setBold(true);
    font2.setColor(Color.BLACK);
    font2.setSize(10);
    categoryAxis.getTitle().setFont(font2);

    //Set properties of valueaxis title
    ValueAxis valueAxis = chart.getValueAxis();
    valueAxis.getTitle().setText("Tensile Strength");
    Font font3 = new Font();
    font3.setBold(true);
    font3.setColor(Color.BLACK);
    font3.setSize(10);
    valueAxis.getTitle().setFont(font3);
    valueAxis.getTitle().setRotation(90);
  }
}

